#!/usr/bin/python

import os, sys, struct
from sys import argv
from stat import *

newfont_bin = \
"\xfe\xb4\x00\xb5\x01\xbc\x0b\xa1\x0a\x68\x00\x2a\x05\xd0\x90\x42" \
"\x01\xd0\x08\x31\xf8\xe7\x48\x68\x02\xe0\x1f\x20\x81\x02\x40\x18" \
"\xfe\xbc\x09\x90\x89\xb0\x14\x1c\x00\x20\x06\x90\x00\x48\x00\x47" \
"\x11\x32\x00\x08\xf3\x44\x00\x08\x01\x00\x00\x00\x13\x45\x00\x08" \
"\x22\x1c\x00\x00\x07\x36\x00\x08\x01\x00\x00\x00\xe3\x0f\x00\x08" \
"\x01\x00\x00\x00\xf7\x0f\x00\x08\xff\x7f\x00\x00\x5b\x5b\x00\x08" \
"\x01\x00\x00\x00\x6d\x62\x00\x08\x01\x00\x00\x00\xd9\x67\x00\x08" \
"\x01\x00\x00\x00\xfb\x6a\x00\x08\x01\x00\x00\x00\x87\x61\x00\x08" \
"\xff\x7f\x00\x00\xc9\x66\x00\x08\xff\x7f\x00\x00\x27\x6a\x00\x08" \
"\xff\x7f\x00\x00\x9d\x5f\x00\x08\xff\x7f\x00\x00\x91\x65\x00\x08" \
"\xff\x7f\x00\x00\xef\x68\x00\x08\xff\x7f\x00\x00\x39\x94\x00\x08" \
"\xff\x7f\x00\x00\xa3\x94\x00\x08\x01\x00\x00\x00\xb7\xb5\x00\x08" \
"\xff\x7f\x00\x00\x85\xb6\x00\x08\xff\x7f\x00\x00\x0b\xb7\x00\x08" \
"\xff\x7f\x00\x00\xe1\x50\x00\x08\x01\x00\x00\x00\xf5\x50\x00\x08" \
"\x22\x1c\x00\x00\x65\x52\x00\x08\x01\x00\x00\x00\x79\x52\x00\x08" \
"\x22\x1c\x00\x00\x00\x00\x00\x00"

def readfile(name):
	try:
		fd = open(name, "rb")
		contents = fd.read()
		fd.close()
	except IOError:
		print "Error reading", name
		sys.exit(1)
	return contents

def writefile(name, contents):
	try:
		fd = open(name, "wb")
		fd.write(contents)
		fd.close()
	except IOError:
		print "Error writing", name
		sys.exit(1)

def convertRGB(rgb):
	if len(rgb) != 7 or rgb[0] != "#":
		print "Invalid color:", rgb
		sys.exit(1)
	return ((convertHex(rgb[1:3])>>3)&0x1f) + \
					(((convertHex(rgb[3:5])>>3)&0x1f)<<5) + \
					(((convertHex(rgb[5:7])>>3)&0x1f)<<10)

def validHex(h):
	allhex = "0123456789abcdefABCDEF"
	return h in allhex

def intHex(h):
	return "0123456789abcdef".find(h.lower())

def convertHex(h):
	if len(h) != 2 or not validHex(h[0]) or not validHex(h[1]):
		print "Invalid hexidecimal number", h
		sys.exit(1)
	return intHex(h[0])*16 + intHex(h[1])

def parsecolors():
	colors = {}
	colorstxt = readfile("colors.txt").splitlines()
	for f in colorstxt:
		dewhited = f.strip()
		if dewhited and dewhited[0] != "#":
			split = dewhited.partition("=")
			if split[1] != "=":
				print "Invalid line:", f
				sys.exit(1)
			name = split[0].lower()
			color = convertRGB(split[2])
			if color == 0: # prevent transparencies
				color = 1
			colors[name] = color
	return colors
	
def insertfix(file, address, color):
	return file[0:address] + struct.pack("<H", color) + file[address+2:]

def insertfixbyte(file, address, byte):
	return file[0:address] + struct.pack("B", byte) + file[address+1:]

def insertfixbytes(file, address, bytes):
	for b in bytes:
		file = insertfixbyte(file, address, b)
		address += 1
	return file

if __name__ == "__main__":
	EZFLA_UP="ezfla_up.bin"

	ezflaup = readfile(EZFLA_UP)

	if len(ezflaup) == 1911424:
		ezflaup = ezflaup + newfont_bin

	colors = parsecolors()

	#SHIFT_FILEMANAGER_BACKGROUND_DOWN="0x5f30=0x18"
	ezflaup = insertfixbytes(ezflaup, 0x5f30, [0x18])
	#SHIFT_FILEMANAGER_HEADER_DOWN="0x5eac=0x18"
	ezflaup = insertfixbytes(ezflaup, 0x5eac, [0x18])
	#SHIFT_ICONS_LEFT="0x112a=0x04"
	ezflaup = insertfixbytes(ezflaup, 0x112a, [0x04])
	#FIX_BACKGROUND_CHECKS="0x390a=0xf0 0x38ee=0xa0"
	ezflaup = insertfixbytes(ezflaup, 0x390a, [0xf0])
	ezflaup = insertfixbytes(ezflaup, 0x38ee, [0xa0])
	#FIX_DIALOG_SHADOW="0x44e4=0x37 0x4504=0x36"
	ezflaup = insertfixbytes(ezflaup, 0x44e4, [0x37])
	ezflaup = insertfixbytes(ezflaup, 0x4504, [0x36])
	#FIX_TIME_SHADOW="0x50d6=0x44 0x50ea=0x43"
	ezflaup = insertfixbytes(ezflaup, 0x50d6, [0x44])
	ezflaup = insertfixbytes(ezflaup, 0x50ea, [0x43])
	#FIX_LOAD_SHADOW="0x525a=0x44 0x526e=0x43"
	ezflaup = insertfixbytes(ezflaup, 0x525a, [0x44])
	ezflaup = insertfixbytes(ezflaup, 0x526e, [0x43])
	#ICONS_TRANSPARENCY_UPPER_LEFT_CORNER="0x3950=0x00,0x23,0xfb,0x5a"
	ezflaup = insertfixbytes(ezflaup, 0x3950, [0x00,0x23,0xfb,0x5a])

	#DIALOG_BACKGROUND="0xbaa=0xc0,0x46 0x5196=0xc0,0x46"
	ezflaup = insertfixbytes(ezflaup, 0x0baa, [0xc0,0x46])
	ezflaup = insertfixbytes(ezflaup, 0x5196, [0xc0,0x46])

	#new_font_stub.bin
	NEW_FONT_STUB_BIN="0x3208=0x00,0x4c,0x20,0x47,0x81,0x2a,0x1d,0x08"
	ezflaup = insertfixbytes(ezflaup, 0x3208, [0x00,0x4c,0x20,0x47,0x81,0x2a,0x1d,0x08])

	ezflaup = insertfix(ezflaup, 0x5e68, colors["file_selection_bar"])
	ezflaup = insertfix(ezflaup, 0x62c4, colors["file_selection_bar"])
	ezflaup = insertfix(ezflaup, 0x6754, colors["file_selection_bar"])
	ezflaup = insertfix(ezflaup, 0x6b74, colors["file_selection_bar"])

	ezflaup = insertfix(ezflaup, 0x9810, colors["settings_selection_bar"])

	ezflaup = insertfix(ezflaup, 0x0be8, colors["dialog_background"])
	ezflaup = insertfix(ezflaup, 0x1004, colors["dialog_background"])
	ezflaup = insertfix(ezflaup, 0x51e0, colors["dialog_background"])
	ezflaup = insertfix(ezflaup, 0x7b5c, colors["dialog_background"])

	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 0, colors["dialog_title_shadow"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 1, colors["dialog_title_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 2, colors["dialog_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 3, colors["version_shadow"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 4, colors["version_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 5, colors["file_selected_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 6, colors["file_selected_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 7, colors["file_selected_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 8, colors["file_selected_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 9, colors["file_unselected_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 10, colors["file_unselected_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 11, colors["file_unselected_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 12, colors["file_unselected_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 13, colors["file_unselected_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 14, colors["file_unselected_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 15, colors["settings_unselected_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 16, colors["settings_selected_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 17, colors["text_viewer_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 18, colors["text_viewer_text"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 19, colors["text_viewer_text"])
	# time
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 20, colors["dialog_title_shadow"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 21, colors["dialog_title_text"])
	# load
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 22, colors["dialog_title_shadow"])
	ezflaup = insertfix(ezflaup, 0x1d2ab8 + 8 * 23, colors["dialog_title_text"])

	writefile(EZFLA_UP, ezflaup)
